Author : Anurag Mukkara (anurag_m@csail.mit.edu)
Date : 30 Jan 2016

A simple pintool that profiles misses out of an 128KB private cache.
Filtering accesses using a private cache greatly reduces the number of
accesses to the profiler, and is representative of the access stream that
a last-level cache(LLC) would see.

This tool was developed as part of Whirltool profiling framework described in
Whirlpool:Improving Dynamic Cache Management with Static Data Classification.
Anurag Mukkara, Nathan Beckmann, Daniel Sanchez, ASPLOS-21, April 2016.
See Section 4 of the paper for a brief introduction into Whirltool framework.

Different kinds of profilers can be used. There are 2 types of profilers
currently implemented:
1)PageAccess : Count the number of accesses to different pages.
3)Miss Curve: Collects miss curves(number of misses at different LLC sizes)

The memory footprint of an application can be divided into various
memory regions(which we call shares), and the accesses to each
region can be profiled independently.
The memory regions can be non-contiguous and are tracked at page granularity.
For example, one can divide an application's data into two regions and
get the miss curves for each region.
Some profilers like missCurveProfiler collect miss curve for each epoch, where
and epoch could be say, 50 millions instructions, long.

To create memory regions one needs to special assembly instructions called magic ops.
However, if you are using this profiler in conjunction with poolMalloc library,
the memory regions will be automatically managed by poolMalloc.

Many system parameters like private cache size, LLC cache size, umon sampling rate etc.
are defined in llcProfiler.h

There is also another pinTool(inscount) that can be used to skip specific number of
instructions(say 20 Billion) during program initialization. This comes handy when
you do want profile the entire application but want to focus on sufficienlty long
region(say 10 Billion) after the initial fast-forward.

Files:
cache.h
Implements a set-associate cache.

memoryAccessProfiler.cpp
Main script that interfaces with pin and profiling classes.

stats.h
Handles stats output in HDF5 format.

llcProfiler.h
All the profiler classes are defined here.

utility_monitos.h
Implements UMONs.

inscount.cpp
Pin tool that counts number of instructions.

hash.h, mtrand.h
Misc files used in profiler implementations.

Usage:
Build the tools by running 'make'.
You will need to setup the paths to PIN and ARMADILLO installation directories
in Makefile(PIN_PATH, ARMADILLO_PATH).

inscount.so
pin -t obj-intel64/inscount.so -start 20000000000 -length 10000000000 (extra_pintool_arguments) -- application application-arguments

memoryAccessProfiler.so
pin -t obj-intel64/memAccessProfiler.so -start 20000000000 -length 10000000000 (extra_pintool_arguments) -- application application-arguments

Acknowledgments:
The implementations of utility monitors is adapted
from zsim simulator(zsim.csail.mit.edu)
