/** $lic$
 * Copyright (C) 2015-2016 by Massachusetts Institute of Technology
 *
 * This file is part of Whirltool.
 *
 * Whirltool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * If you use this software in your research, we request that you reference
 * the Whirlpool paper ("Whirlpool: Improving Dynamic Cache Management with Static Data
 * Classification", Mukkara, Beckmann and Sanchez, ASPLOS-21, April 2016) as the
 * source in any publications that use this software, and that you send us a
 * citation of your work.
 *
 * Whirltool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Whirltool.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/
#ifndef STATS_H_
#define STATS_H_

#include <hdf5.h>
#include <hdf5_hl.h>
#include <armadillo>

void createAndSaveDataSpace(char* filename, uint32_t rank, hsize_t* dimsf,
                            uint32_t* data, char* dataset_name) {
    hid_t file, dataset;       /* file and dataset handles */
    hid_t datatype, dataspace; /* handles */

    /*
    * Create a new file using H5F_ACC_TRUNC access,
    * default file creation properties, and default file
    * access properties.
    */
    file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

    dataspace = H5Screate_simple(rank, dimsf, NULL);

    /*
     * Define datatype for the data in the file.
     * We will store little endian INT numbers.
     */
    datatype = H5Tcopy(H5T_NATIVE_UINT);

    /*
     * Create a new dataset within the file using defined dataspace and
     * datatype and default dataset creation properties.
     */
    dataset = H5Dcreate2(file, dataset_name, datatype, dataspace, H5P_DEFAULT,
                         H5P_DEFAULT, H5P_DEFAULT);

    /*
     * Write the data to the dataset using default transfer properties.
     */
    H5Dwrite(dataset, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data);

    /*
     * Close/release resources.
     */
    H5Sclose(dataspace);
    H5Tclose(datatype);
    H5Dclose(dataset);
    H5Fclose(file);

    return;
}

void dump3DMat(char* filename, arma::Cube<uint32_t> input, char* dataset_name) {
    // Describe the size of the array
    hsize_t dimsf[3]; /* dataset dimensions */
    dimsf[0] = input.n_slices;
    dimsf[1] = input.n_cols;
    dimsf[2] = input.n_rows;

    createAndSaveDataSpace(filename, 3, dimsf, input.memptr(), dataset_name);
}

#endif  // STATS_H_
